#include "exam.h"
#include "lcd.h"
#include "uart.h"
#include "sensors.h"

#include "FreeRTOS.h"
#include "task.h"
#include "timers.h"

#include "gpio.h"
#include <string.h>

void writeLCD(unsigned addr, const char *str) {
	LCD_Enqueue(LCD_INSTRUCTION, SET_DDRAM_ADDRESS_INSTRUCTION | addr);
	for (size_t i = 0; i < strlen(str); ++i) {
		LCD_Enqueue(LCD_DATA, str[i]);
	}
}

void writeUART(char *str) {
	for (size_t i = 0; i < strlen(str); ++i) {
		UART_Transmit(str[i]);
	}
}

void intToStr(int num, char *str, int maxLen) {
	for (int i = 0; i < maxLen; ++i) {
		str[i] = ' ';
	}
	int index = maxLen - 1;
	if (num == 0) {
		str[maxLen - 1] = '0';
	}
	while (num > 0 && index >= 0) {
		str[index--] = '0' + (num % 10);
		num /= 10;
	}
	str[maxLen] = '\0';
}

float prevTemperatureSensor;
float prevWindvaneSensor;
float prevAnemometerSensor;

void examTask(void *parameters) {
	UNUSED(parameters);
	// Ekvivalentne adrese: 0x80, 0xC0, 0x90
	// Kontroleru se ne mogu poslati adrese sa preko 7 bita
	writeLCD(0x00, "Temper: ");
	writeLCD(0x40, "Azimut: ");
	writeLCD(0x10, "Brzina: ");
	while (1) {
		if (((int) prevTemperatureSensor) != ((int) temperatureSensor)
				|| ((int) prevWindvaneSensor) != ((int) windvaneSensor)
				|| ((int) prevAnemometerSensor) != ((int) anemometerSensor)) {
			prevTemperatureSensor = temperatureSensor;
			prevWindvaneSensor = windvaneSensor;
			prevAnemometerSensor = anemometerSensor;
			char temper[4];
			char azimut[4];
			char brzina[4];
			intToStr((int) temperatureSensor, temper, 3);
			intToStr((int) windvaneSensor, azimut, 3);
			intToStr((int) anemometerSensor, brzina, 3);
			writeLCD(0x08, temper);
			writeLCD(0x48, azimut);
			writeLCD(0x18, brzina);
			writeUART("\b\b\b\b\b\b\b\b\b\b\b");
			writeUART(azimut);
			writeUART("/");
			writeUART(brzina);
			writeUART("/");
			writeUART(temper);
		}
		vTaskDelay(pdMS_TO_TICKS(200));
	}
}

void ledTimer(TimerHandle_t xTimer) {
	UNUSED(xTimer);
	if (anemometerSensor < 50) {
		HAL_GPIO_WritePin(GPIOA, GPIO_PIN_13, GPIO_PIN_RESET);
	} else {
		HAL_GPIO_TogglePin(GPIOA, GPIO_PIN_13);
	}
}

void examInit() {
	LCD_Init();
	UART_Init();
	Sensors_Init();
	xTaskCreate(examTask, "examTask", 128, NULL, 2, NULL);
	TimerHandle_t timer = xTimerCreate("ledTimer", pdMS_TO_TICKS(500), pdTRUE,
	NULL, ledTimer);
	xTimerStart(timer, portMAX_DELAY);
}
