#include "lcd.h"
#include "gpio.h"

#define LCD_ENABLE_BIT 0x04

QueueHandle_t LCD_QueueHandle;
LCD_Command cmd;

void LCD_Write(LCD_Register reg, uint8_t data) {
	GPIOC->ODR = (data << 3) | reg;
	GPIOC->ODR |= LCD_ENABLE_BIT;
	GPIOC->ODR &= ~LCD_ENABLE_BIT;
}

void LCD_WriteCommand(LCD_Register reg, uint8_t data) {
	LCD_Write(reg, data >> 4);
	LCD_Write(reg, data);
	vTaskDelay(pdMS_TO_TICKS(2));
}

void LCD_Task(void *parameters) {
	UNUSED(parameters);
	LCD_Write(LCD_INSTRUCTION, (FUNCTION_INSTRUCTION) >> 4);
	vTaskDelay(pdMS_TO_TICKS(2));
	LCD_WriteCommand(LCD_INSTRUCTION, FUNCTION_INSTRUCTION | FUNCTION_2_LINES);
	LCD_WriteCommand(LCD_INSTRUCTION, DISPLAY_INSTRUCTION | DISPLAY_ON);
	LCD_WriteCommand(LCD_INSTRUCTION,
	ENTRY_MODE_INSTRUCTION | ENTRY_MODE_INC_CURSOR);
	LCD_WriteCommand(LCD_INSTRUCTION, RETURN_HOME_INSTRUCTION);
	while (1) {
		xQueueReceive(LCD_QueueHandle, &cmd, portMAX_DELAY);
		LCD_WriteCommand(cmd.reg, cmd.data);
	}
}

void LCD_Enqueue(LCD_Register reg, uint8_t data) {
	cmd.reg = reg;
	cmd.data = data;
	xQueueSendToBack(LCD_QueueHandle, &cmd, portMAX_DELAY);
}

void LCD_Init() {
	xTaskCreate(LCD_Task, "LCD_Task", 128, NULL, 5, NULL);
	LCD_QueueHandle = xQueueCreate(64, sizeof(LCD_Command));
}
