#include "sensors.h"

#include "FreeRTOS.h"
#include "task.h"

#include "adc.h"
#include "gpio.h"
#include "tim.h"

#define MAX_VOLTAGE 5.0
#define MAX_RESOLUTION 4096

float temperatureSensor = 0.0;
float anemometerSensor = 0.0;
float windvaneSensor = 0.0;
int fetchingTemperature;
unsigned overflowCounter = 0;
int previousTicks = 0;

TaskHandle_t Sensors_TaskHandle;

void Sensors_Task(void *parameters) {
	UNUSED(parameters);
	while (1) {
		fetchingTemperature = 1;
		HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
		HAL_ADC_Start_IT(&hadc1);
		ulTaskNotifyTake(pdTRUE, portMAX_DELAY);

		fetchingTemperature = 0;
		HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
		HAL_ADC_Start_IT(&hadc1);
		ulTaskNotifyTake(pdTRUE, portMAX_DELAY);

		vTaskDelay(pdMS_TO_TICKS(100));
	}
}

float adcToWindvane(int adcValue) {
	switch (adcValue) {
	case 952:
		return 0.0;
	case 2471:
		return 22.5;
	case 2250:
		return 45.0;
	case 3760:
		return 67.5;
	case 3723:
		return 90.0;
	case 3831:
		return 112.5;
	case 3356:
		return 135.0;
	case 3589:
		return 157.5;
	case 2946:
		return 180.0;
	case 3116:
		return 202.5;
	case 1575:
		return 225.0;
	case 1698:
		return 247.5;
	case 315:
		return 270.0;
	case 786:
		return 292.5;
	case 547:
		return 315.0;
	case 1285:
		return 337.5;
	default:
		return 0.0;
	}
}

void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef *hadc) {
	if (hadc->Instance != hadc1.Instance) {
		return;
	}
	uint32_t value = HAL_ADC_GetValue(hadc);
	if (fetchingTemperature) {
		temperatureSensor =
				((float) value) * MAX_VOLTAGE * 100 / MAX_RESOLUTION;
	} else {
		windvaneSensor = adcToWindvane(value);
	}
	xTaskNotifyGive(Sensors_TaskHandle);
}

void HAL_TIM_IC_CaptureCallback(TIM_HandleTypeDef *htim) {
	if (htim->Instance != htim1.Instance) {
		return;
	}
	float ticks = htim->Instance->CCR1 + overflowCounter * 65536
			- previousTicks;
	previousTicks = htim->Instance->CCR1;
	overflowCounter = 0;
	anemometerSensor = 8000000.0 / ticks * 2.4;
}

void Sensors_Init() {
	HAL_TIM_Base_Start_IT(&htim1);
	HAL_TIM_IC_Start_IT(&htim1, TIM_CHANNEL_1);
	xTaskCreate(Sensors_Task, "Sensors_Task", 128, NULL, 5,
			&Sensors_TaskHandle);
}
